PAGE 64,132
NAME Printer_Driver_HP_Desk_Jet
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Last Modification Date: March 3, 1988
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; DECLARATIONS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Save_DS     MACRO
        push    DS                  ;save old DS
        push    CS
        pop     DS                  ;establish DS addressability
        assume  DS:PRINTER
            ENDM

Restore_DS  MACRO
        pop     DS                  ;restore DS
        assume  DS:nothing
            ENDM

;error codes
        E_OK                    equ  0
        E_Invalid_Function      equ  1
        E_File_Not_Found        equ  2
        E_Path_Not_Found        equ  3
        E_No_Handles_Left       equ  4
        E_Access_Denied         equ  5
        E_Invalid_Handle        equ  6
        E_Bad_Control_Block     equ  7
        E_Memory_Limit          equ  8
        E_Bad_Memory_Address    equ  9
        E_Bad_Environment       equ 10
        E_Bad_Format            equ 11
        E_Bad_Access_Code       equ 12
        E_Bad_Data              equ 13
        E_Disk_Full             equ 14
        E_Bad_Drive_Specified   equ 15
        E_Current_Dir_Remove    equ 16
        E_Not_Same_Device       equ 17
        E_No_More_Files         equ 18

STACK   segment stack   'STACK'
STACK   ends

PRINTER segment public  'PRINTER'
        assume  CS:PRINTER
        assume  DS:nothing

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Printer_Driver
;       interface to printer driver is made with a series of far calls
;       all parameters are passed via registers, the specific registers are
;       detailed for the sub-command
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Printer_Driver  proc far
        jmp     Initialize_Driver
        jmp     Get_Printer_Information
        jmp     Get_Driver_Buffer_Address
        jmp     Initialize_Sheet
        jmp     Completed_Sheet
        jmp     Page_Break
        jmp     Output_Printhead_Pass

Sign_On_Message  db 'HP DESKJET (300 dpi) (Letter Paper) Printer Driver  V4.20',0Dh,0Ah,'$'
COPYRIGHT_NOTICE db '(C) Copyright 1987 OrCAD, Inc.'
                 db '  ALL RIGHTS RESERVED.','$'

Initialize_Driver:                      ;initialize the interface
        Save_DS
; display the program title, revision and copyright notice
        mov     DX,offset(Sign_On_Message)
        mov     AH,09h
        int     21h                     ;print Sign_On_Message
        lea     DX,Copyright_Notice
        mov     AH,9
        int     21h                     ; print message
; return unused memory to the pool
        mov     AX,CS                   ;segment address of this overlay
        mov     ES,AX                   ;recall segment address
        mov     BX,offset(End_Of_File_Location)
        mov     CL,4
        shr     BX,CL
        add     BX,2
        mov     AH,4Ah                  ;modify allocated memory blocks
        int     21h
        Restore_DS
        ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Get_Printer_Information
;       pass back a pointer to the Printer_Information structure in ES:BX
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Get_Printer_Information:
        Save_DS
        mov     BX,CS
        mov     ES,BX
        mov     BX,offset(Printer_Information)
        Restore_DS
        ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Get_Driver_Buffer_Address
;       pass back a pointer to the Driver_Buffer in ES:BX to be used for
;       future calls to Output_Printhead_Pass
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Get_Driver_Buffer_Address:
        Save_DS
        mov     BX,CS
        mov     ES,BX
        mov     BX,offset(Driver_Buffer)
        Restore_DS
        ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Initialize_Sheet
;       pass in HANDLE in BX, destination of this sheet, already opened
;       HANDLE will be checked here to ensure binary mode
;       initial commands written to the HANDLE
;       pass back STATUS in AL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Initialize_Sheet:                       ;initialize the driver for a new sheet
        Save_DS
        mov     Handle,BX               ;save handle for later routines
; check handle
        mov     AX,4400h
        int     21h                     ;get information on handle
        jc      IS_Exit                 ;if error, exit
        test    DL,80h                  ;is handle for a device?
        jz      IS_Write
        or      DX,0000000000100000b    ;force binary mode
        xor     DH,DH
        mov     AX,4401h                ;set device information
        int     21h
        jc      IS_Exit                 ;if error, exit
IS_Write:
        lea     DX,Enter_Graphics       ; transfer the ENTER GRAPHICS command
        mov     BX,Handle
        mov     CX,24
        call    Write
IS_Exit:
        Restore_DS
        ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Completed_Sheet
;       write to HANDLE at the end of the sheet printing
;       pass STATUS back in AL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Completed_Sheet:
        Save_DS
        lea     DX,Leave_Graphics       ;transfer the LEAVE GRAPHICS command
        mov     BX,Handle
        mov     CX,5
        call    Write
        Restore_DS
        ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Page_Break
;       suspend printing on the current page, send form feed, restore printing
;       return STATUS in AL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Page_Break:
        Save_DS
        lea     DX,Leave_Graphics       ;transfer the LEAVE GRAPHICS command
        mov     BX,Handle
        mov     CX,29
        call    Write
        Restore_DS
        ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Output_Printhead_Pass
;       convert the Driver_Buffer into a format that the printer will use
;       remove all trailing zero bytes to reduce print and transfer time
;       return STATUS in AL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Output_Printhead_Pass:
        Save_DS
; reduce the transfer count to eliminate trailing zeros
        mov     CX,Driver_Buffer_Size
        mov     AX,DS
        mov     ES,AX
        mov     BX,CX
        dec     BX
        lea     DI,Driver_Buffer[BX]
        xor     AL,AL
        std
        rep     scasb
; convert the value in CX to a printer byte count
        inc     CX
        mov     Row_Count,CX
        mov     AX,CX
        xor     DX,DX
        mov     CL,10
        div     CL
        add     AH,'0'
        mov     Printer_Code[5],AH
        mov     AH,AL
        xor     AH,AH
        div     CL
        add     AH,'0'
        mov     Printer_Code[4],AH
        mov     AH,AL
        xor     AH,AH
        div     CL
        add     AH,'0'
        mov     Printer_Code[3],AH
; transfer the graphics row leadin
        mov     CX,7
        lea     DX,Printer_Code
        call    Write
        jne     OPP_Exit
; transfer the graphics row
        mov     CX,Row_Count
        lea     DX,Driver_Buffer
        call    Write
        jne     OPP_Exit
OPP_Exit:
        Restore_DS
        ret

Printer_Driver  endp

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Write
;       transfer bytes to the HANDLE
;       pass in address of data in DS:DX
;       pass in byte count in CX
;       return STATUS in AL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Write           proc near

        assume  DS:nothing

        mov     BX,CS:Handle
        jcxz    Write_Exit
IFDEF NEC
        cmp     BX,4                            ;is the handle the printer?
        je      Write_PRN                       ;if yes, jump
ENDIF
        mov     AH,40h
        push    CX                      ;save count for later check
        int     21h
        pop     CX                      ;recall count
        jc      Write_Exit
        cmp     AX,CX                   ;compare written count
        mov     AL,E_Disk_Full          ;if written count <> write count
        jne     Write_Exit              ;   then disk must be full
        mov     AL,E_OK
Write_Exit:
        ret
IFDEF NEC
Write_PRN:
        push    DS
        mov     BX,DS
        mov     ES,BX                           ;address of buffer is ES
        mov     BX,DX                           ;address of buffer is BX
        mov     AH,30h                          ;block write
        int     1Ah
        shr     AH,1
        shr     AH,1                            ;if error, carry is now set
        pop     DS
        mov     AL,E_Access_Denied
        jc      Write_Exit
        mov     AL,E_OK
        jmp     Write_Exit
ENDIF
Write           endp

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; DATA AREA
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Handle                  dw      ?
Row_Count               dw      ?
Leave_Graphics          db      1Bh,'*rB'
                        db      0Ch
Enter_Graphics          db      1Bh,'*rB'
                        db      1Bh,'*t300R'
                        db      1Bh,'*r2400S'
                        db      1Bh,'*r0A'
Printer_Code            db      1Bh,'*b000W'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Driver_Buffer           db      300 dup (?)     ;buffer to pass bits into
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Printer_Information     dw      2400            ;bits per line (narrow paper)
                        dw      2400            ;bits per line (wide paper)
                        dw      300             ;bits per inch (horizontal)
                        dw      300             ;bits per inch (vertical)
                        dw      1               ;lines per printhead pass
                        dw      300*10          ;printhead passes per page
;<<<NOTE>>> set the printhead passes per page to 0 if continuous paper
Driver_Buffer_Size      dw      300             ;Driver_Buffer size (in bytes)
Driver_Buffer_Row_Size  dw      300             ;Driver_Buffer size (in bytes)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; the following must be the last declaration so that the memory size used by
; the driver can be reduced
End_Of_File_Location    db      ?

PRINTER ends
        end
